      program asktit

c     Asks for and encodes the title of a latex2html document.  Puts the
c     encoded title in file l2htitle.rht.  If the user aborts, or an
c     error occurred the return error level will be 1.

c     Under Windows, compile this file using the gnu Fortran compiler as:
c        g77 -o ..\var\asktitle.exe asktitle.f lib0.f congnuc.f congnuc2.c
c     Under Unix, compile this file using the gnu Fortran compiler as:
c        g77 -o ..\var\asktitle.exe asktitle.f lib0.f congnuw.f
c        g77 -o ..\var\asktitle2 asktitle.f lib0.f congnu.f

c     avoid typos
      implicit none

c     whether l2htitle.rht exists
      logical exists

c     title line
      integer titd
      parameter (titd=1024)
      character*(titd) tit
      integer titl

c     number of coded characters
      integer codmax
      parameter (codmax=5)
c     coded  character index
      integer cod
c     coded characters
      character*(codmax) codchr
      data codchr/'&<>"\\'/
c     encoded strings
      character*6 codstr(codmax)
      data codstr/'&amp;','&lt;','&gt;','&quot;','&#92;'/
c     length of the strings
      integer codlt

c     input line
      character*80 line
      integer linelt

c     saved line
      character*80 savlin
      integer savllt

c     character indices
      integer ic,ic2

c     character
      character*1 chr

c executable

 10   format(a,a,a)

c     allow I/O to use units 7 and on
      call libini(7)

c     set the error exit status to 1
      call errset(1)

c     open l2htitle.rht and read the title
      titl=0
      inquire(file='l2htitle.rht',exist=exists,err=910)
      if(.not.exists)goto 190
      open(1,file='l2htitle.rht',status='old',err=920)
      read(1,150,err=930,end=170)tit
 150  format(a1024)
      if(tit(titd-50:titd).ne.' ')call out(
     & '** Warning: title too long in l2htitle.rht')
      do 160 ic=titd,1,-1
         titl=ic
         if(tit(ic:ic).gt.' ')goto 180
 160  continue
      titl=0
 170  call out('** Warning: missing title in l2htitle.rht')
 180  close(1,err=940)
 190  continue

c     initialize the prompt line
      line='Title: '
      linelt=7

c     decode the default and add to the line
      ic=0
c     next encoded character
 200  ic=ic+1
      if(ic.gt.titl)goto 290
      if(linelt.ge.79)then
         line(77:79)='...'
         goto 290
      endif
      chr=tit(ic:ic)
c     decode all known sequences
      if(chr.eq.'&')then
         do 250 cod=1,codmax
            codlt=index(codstr(cod),';')
            ic2=ic+codlt-1
            if(tit(ic:min(titl,ic2)).eq.codstr(cod)(1:codlt))then
               ic=ic2
               linelt=linelt+1
               line(linelt:linelt)=codchr(cod:cod)
               goto 200
            endif
 250     continue
      endif
c     just copy the character to the decoded string
      linelt=linelt+1
      line(linelt:linelt)=chr
      goto 200
 290  continue

c     save
      savlin=line(1:linelt)
      savllt=linelt

c     Write basic help
      call out(' ')
      if(titl.gt.0)then
         call out2('Please enter a brief running title, '//
     &    'or hit [Return] to leave the current')
         call out2('value unchanged.  Enter a solitary question '//
     &    'mark for help.')
      else
         call out2('Please enter a brief running title, or '//
     &    'a solitary question mark for help.')
      endif

 400  line=savlin
      call in2(line,linelt)

      if(line(1:linelt).eq.' ')then
         call out2('** The running title cannot be blank.')
         goto 400
      endif

      if(linelt.eq.1 .and. line(1:1).eq.'?')then
         call out2(' ')
         if(titl.gt.0)then
            call out2('To leave the current value, shown in '//
     &       'the prompt, unchanged, just hit [Return].')
            call out2(' ')
         endif
         call out2('The text you can enter is limited in '//
     &    'length due to program limitations.')
         call out2('Normally, any text you want to enter '//
     &    'for a running title is short, but if not,')
         call out2('you will have to very carefully edit '//
     &    'file l2htitle.rht, maintaining its layout')
         call out2('unchanged.  In particular, do not split '//
     &    'the line.  (Windows Notepad will not')
         call out2('split lines permanently by itself, but '//
     &    'be sure to put the filename between')
         call out2('double quotes when you save, or you '//
     &    'may save l2htitle.rht.txt instead.)  Then')
         call out2('rerun l2h or makel2h.')
         call out2(' ')
         call out2('To add multiple blank spaces, write '//
     &    'each blank space as &nbsp; or they will')
         call out2('be ignored.  The string &nbsp;&nbsp;&nbsp;&nbsp; '//
     &    'gives four spaces.')
         call out2(' ')
         call out2('Special characters such as &copy; for '//
     &    'the copyright character can be added')
         call out2('from the entities.html list in the subfolder '//
     &    'files of the l2hsup folder.')
         call out2('Do not try to add HTML tags: browsers '//
     &    'treat titles as plain text.')
         call out2(' ')
         goto 400
      endif

c     exit if the default is unchanged
      if(line(1:linelt).eq.savlin(8:max(8,savllt)))goto 800

c     encode the input
      titl=0
      ic=0
c     next character from the input string
 500  ic=ic+1
      if(ic.gt.linelt)goto 590
      chr=line(ic:ic)
      cod=index(codchr,chr)
      if(cod.gt.0)goto 520
c     put the character in the encoded string as is
 510  titl=titl+1
      if(titl.gt.titd)call out('*** Encoded title is too long!')
      tit(titl:titl)=chr
      goto 500
c     must decide whether an & is already an encoded sequence or not
 520  if(chr.eq.'&')then
         ic2=index(line(ic:linelt),';')
         if(ic2.le.2)goto 530
         ic2=ic2+ic-1
         if(index(line(ic+1:ic2),'&').gt.0)goto 530
         if(index(line(ic+1:ic2),' ').gt.0)goto 530
         if(index(line(ic+1:ic2),char(9)).gt.0)goto 530
         goto 510
      endif
c     encode the character
 530  codlt=index(codstr(cod),';')
      if(titl+codlt.gt.titd)call out('*** Encoded string is too long!')
      tit(titl+1:titl+codlt)=codstr(cod)
      titl=titl+codlt
      goto 500
 590  continue

c     create a new l2htitle.rht batch file
      call opennw(1,'l2htitle.rht')
      write(1,10,err=960)tit(1:titl)
      close(1,err=960)

 800  call exit(0)

 910  call out('*** Error looking for l2htitle.rht?!')
 920  call out('*** Error accessing l2htitle.rht?!')
 930  call out('*** Error reading l2htitle.rht?!')
 940  call out('*** Error closing l2htitle.rht?!')

 960  call out('*** Unable to write to l2htitle.rht; '//
     & 'check disc space, file attributes!')

      end
