      program choose

c     Ask the user to press a key from a list

c     Usage: choose choices "Prompt"
c        or: choose akey "Prompt"
c        or: choose anykey "Prompt"

c     Operation of the first form above:

c     The returned error level will be the character position in
c     choices.  Errorlevel 0 indicates some error, including the user
c     pressing Ctrl-C or Esc.

c     If choices contains no uppercase or no lowercase characters, the
c     query will be case-insensitive.

c     The quotes are stripped from the prompt and if there is no colon,
c     it will be appended.  The prompt to colon can be 77 characters
c     long.  A default character must be one space separated from the
c     colon.

c     There are exceptions to the above if choices is "yn".  In that
c     case, if there is no colon, the string "? [y/n]: " is appended.
c     If there is a colon immediately followed by a y or n, then
c     "? [y/n]: y" or "? [y/n]: n" replaces the colon and y or n.

c     Operation of choose anykey

c     The return status will be 0 even for program errors (!).

c     Operation of choose akey

c     The return status will be zero, unless the user presses [Esc] or
c     [Ctrl-C] or there are program errors, where it will be 1.

c     Generic version can be made with:
c     g77 -o ..\var\choose.exe choose.f lib0.f argssun.f congnuw.f
c     or on Unix:
c     g77 -o ../var/choose.exe choose.f lib0.f argssun.f congnuw.f
c     Version for DOS, with direct keyboard access:
c     g77 -o ..\var\choose.exe choose.f lib0.f argssun.f congnuc.f congnuc2.c
c     Version for UNIX, with direct keyboard access
c     g77 -o ../var/choose2 choose.f lib0.f argssun.f congnu.f

c     avoid typos
      implicit none

c local

c     declared number of arguments
      integer argdim
      parameter (argdim=2)
c     actual number of arguments
      integer argmx
c     arguments
      character*80 argt(argdim)
      integer argtl(argdim)

c     input line
      character*80 line
      integer linelt
      integer icol

c     what characters are good
      integer good

c     upper case and lowe case characters
      integer iup,ilo

c     character
      character*1 chr

c     character index
      integer i

c executable

c     allow I/O to use units 7 and on
      call libini(7)

c     set the error exit status to 0
      call errset(0)

c     get the command line arguments
      call args(argdim,argmx,argt,argtl)
      if(argmx.ne.2)goto 990

c     set the accepted characters
      if(argt(1)(1:argtl(1)).eq.'akey')then

         call errset(1)
         good=1

      else if(argt(1)(1:argtl(1)).eq.'anykey')then

         good=1

      else

c        look for case sensitivity
         ilo=0
         iup=0
         do 100 i=1,argtl(1)
            chr=argt(1)(i:i)
            if(chr.ge.'a' .and. chr.le.'z')ilo=ilo+1
            if(chr.ge.'A' .and. chr.le.'Z')iup=iup+1
 100     continue
         good=4
         if(ilo*iup.eq.0)good=5

      endif

c     create the prompt
      if(argt(2)(argtl(2):argtl(2)).eq.'"')argtl(2)=argtl(2)-1
      if(argtl(2).le.0)goto 990
      if(argt(2)(1:1).ne.'"')then
         linelt=argtl(2)
         if(linelt.gt.79)call out('*** choose: prompt too long!')
         line=argt(2)(1:argtl(2))
      else
         linelt=argtl(2)-1
         if(linelt.le.0)goto 990
         if(linelt.gt.79)call out('*** choose: prompt too long!')
         line=argt(2)(2:argtl(2))
      endif
      icol=index(line(1:linelt),':')
      if(icol.le.0)then
         if(argt(1)(1:argtl(1)).eq.'yn')then
            if(linelt.gt.69)call out('*** choose: prompt too long!')
            line(linelt+1:linelt+8)='? [y/n]:'
         else
            if(linelt.gt.76)call out('*** choose: prompt too long!')
            line(linelt+1:linelt+1)=':'
         endif
      else
         if(argt(1)(1:argtl(1)).eq.'yn')then
            if(linelt.gt.71)call out('*** choose: prompt too long!')
            if(index('yn',line(linelt:linelt)).le.0)
     &       call out('*** choose: invalid y/n default!')
            line(linelt+8:linelt+8)=line(linelt:linelt)
            line(linelt-1:linelt+6)='? [y/n]:'
         else
            if(icol.gt.77)call out('*** choose: prompt too long!')
            if(line(icol+1:icol+1).ne.' ')
     &       call out('*** choose: colon must be followed by space!')
            if(line(icol+3:80).ne.' ')
     &       call out('*** choose: default must be a single character!')
         endif
      endif

c     get the response by the user
      call inkey(good,argt(1)(1:argtl(1)),line,chr)

c     return the key
      if(good.ge.4)call exit(index(argt(1)(1:argtl(1)),chr))
      call exit(0)

 990  call out2('Usage: choose choices "Prompt"')
      call exit(0)
      end
