      subroutine outcon(nocr,bell,string)

c     Write a string to the screen.
c     If nocr is nonzero, suppress the crlf at the end.
c     If bell is nonzero, will ring the bell.

      implicit none

c input

c     nonzero to suppress the cr-lf
      integer nocr

c     nonzero to write a bell
      integer bell

c     string to write
      character*(*) string

c formats

   10 format(a)
   20 format(a,$)
   30 format(a1,a)
   40 format(a1,a,$)

c executable

      if(nocr.eq.0 .and. bell.eq.0)print10,string
      if(nocr.ne.0 .and. bell.eq.0)print20,string
      if(nocr.eq.0 .and. bell.ne.0)print30,char(7),string
      if(nocr.ne.0 .and. bell.ne.0)print40,char(7),string

      return
      end


      subroutine incon(line)

c     Read a line from the keyboard.

c     Generic version for the GNU compiler.  It DOES NOT allow line
c     editing.  Congnu.f for UNIX or congnuc.f and congnuc2.c for
c     Windows do.

      implicit none

c input/output

c     input: Prompt to use.  Must be terminated by a colon.
c            Any default must follow the colon and a space or question mark
c            (question mark allows a blank input even with a default)
c     output: line read
      character*80 line

c local

c     kept copy of the input line
      character*80 linsav

c     position of the colon, and its original value
      integer icol,icol0

c     length of line
      integer imx

c     character
      character*1 chr

c     character index
      integer i

c executable

c     copy line
      linsav=line

c     find colon
 1    icol=index(linsav,':')
      icol0=icol
      line(icol+1:icol+1)=' '

c     search for default
      imx=79
      do 10 i=79,icol+2,-1
	 if(linsav(i:i).ne.' ')goto 11
	 imx=i-1
   10 continue
   11 continue

c     add default to prompt
      if(imx.le.icol+1)goto 200
      if(imx+4.gt.79)then
	 call outcon(0,1,'*** prompt too long')
	 call errxit
      endif
      line(icol:imx+3)=' ['//linsav(icol+2:imx)//']: '
      icol=imx+2

c     note that none can be entered
      if(linsav(icol0+1:icol0+1).eq.'?')
     & call out2('Enter "none" to clear the current value.')

c     print the line
  200 call outcon(1,0,line(1:icol+1))

c     read keyboard
  300 read(*,310,end=320,err=330)line
  310 format(a80)
      goto 400
  320 call outcon(0,1,'*** INCON: lost keyboard!')
      call errxit
  330 call outcon(0,1,'*** INCON: error during read!')
      call errxit

c     exit on Ctrl-C, refuse inputs with delete or backspace characters
 400  do 410 i=1,80
         chr=line(i:i)
         if(chr.eq.char(3))call errxit
         if(chr.eq.char(8) .or. chr.eq.char(127))then
            call out('** Try deleting characters with Backspace, '//
     &       'Delete, or Ctrl-H')
            line=linsav
            goto 1
         endif
 410  continue
c     substitute default on blank, clear default on none
      if(line.eq.' ')then
         if(imx.ge.icol0+2)line=linsav(icol0+2:imx)
      else if((line.eq.'none' .or. line.eq.' none')
     & .and. linsav(icol0+1:icol0+1).eq.'?')then
         line=' '
      endif

      return
      end


      subroutine inconk(mode,line,ans)

c     Get a key from the keyboard.

c     Generic version for the GNU compiler.  It DOES NOT get the key
c     directly from the keyboard.  Congnu.f for UNIX or congnuc.f and
c     congnuc2.c for Windows do.

      implicit none

c input

c     mode:
c     0 ask for the key
c     1 key was no good, repeat request
c     2 key was OK, clean up the screen as needed
      integer mode

c input/output

c     input: Prompt to use.  Must be terminated by a colon.
c     output: the full user input
      character*80 line

c output

c     the first nonblank character of the user input, if any
      character*1 ans

c local

c     position of the colon
      integer icol

c     default
      character*1 deflt

c     line length
      integer linelt

c executable

c     just return if we are done
      if(mode.eq.2)return

c     get default
      icol=index(line,':')
      deflt=line(icol+2:icol+2)

c     get the response by the user
      call in2(line,linelt)
      ans=line(1:1)

c     apply default, if any
      if(ans.eq.' ' .and. deflt.ne.' ')ans=deflt

      return
      end



      subroutine idle(secs)

c     Wait a number of seconds.

c input

c     number of seconds to wait
      real secs

c local

c     run time
      real sec0,sec

      if(secs.lt.0.)then
	 call outcon(0,1,'*** IDLE: invalid time')
	 call errxit
      endif

      call second(sec0)
  100 call second(sec)
      if(sec.ge.sec0 .and. sec-sec0.lt.secs)goto 100

      return
      end
