      program l2hnew

c     Purpose: determine if a file is newer than another.

c     Usage: l2hnewer file1 file2

c     Normally, file1 is a source of a remake and file2 the target.
c     Remake is needed if the return status is 1.

c     Returns with status 0 if file1 is older (file2 is newer).
c     Returns with status 1 if file2 is older or does not exist.
c     Returns with status 2 if an error occurred.

c     This program was written for the free GNU Fortran compiler g77
c     at www.gnu.org.  To make the l2hnewer.exe executable, use, say:
c     g77 -Wall -o ..\l2hnewer.exe l2hnewer.f

c     avoid typos
      implicit none

c     number of command arguments
      integer iargc
      external iargc
c     argument index
      integer arg
c     argument string
      character*256 argstr(2)
      integer arglt(2)

c     character index
      integer i

c     whether a file exist
      logical exists

c     return status
      integer stat

c     file data block
      integer fdata(13)
c     file dates as integers
      integer date1,date2

c     read command line
      if(iargc().ne.2)then
	 print*,'Usage: l2hnewer file1 file2'
	 call exit(2)
      endif
      do 100 arg=1,2
         argstr(arg)=' '
         call getarg(arg,argstr(arg))
         do 50 i=256,1,-1
            arglt(arg)=i
            if(argstr(arg)(i:i).gt.' ')goto 100
   50	 continue
	 print*,'*** l2hnewer: blank file specification'
	 call exit(2)
 100  continue

c     see whether the files exist
      inquire(file=argstr(1)(1:arglt(1)),exist=exists,err=900)
      if(.not.exists)then
	 print*,'*** l2hnewer: '//argstr(1)(1:arglt(1))//' does not exist'
	 call exit(2)
      endif
      inquire(file=argstr(2)(1:arglt(2)),exist=exists)
      if(.not.exists)call exit(1)

c     open the files
      open(1,file=argstr(1)(1:arglt(1)),status='old',err=900)
      open(2,file=argstr(2)(1:arglt(2)),status='old',err=900)

c     get the file dates
      call fstat(1,fdata,stat)
      if(stat.ne.0)then
         print*,'*** l2hnewer: unable to get data on file1'
         call exit(2)
      endif
      date1=fdata(10)
      call fstat(2,fdata,stat)
      if(stat.ne.0)then
         print*,'*** l2hnewer: unable to get data on file2'
         call exit(2)
      endif
      date2=fdata(10)

c     exit with status 1 if the first file is newer, otherwise 0
      if(date1.gt.date2)call exit(1)
      call exit(0)

c     error exit
 900  print*,'*** l2hnewer: crashed.  Check file specifications.'
      call exit(2)
      end
