      subroutine libini(iounit)

c     This subroutine should be called at the start of any program
c     that uses the other subroutines in this file.

      implicit none

c input

c     Start of I/O unit range reserved for lib.f.   Set this higher than
c     any other I/O unit used by the program.
      integer iounit

c     Note that lib also reserves a common block name for its use, lib_cm

c common

c     I/O units.  They are already reserved if nonzero.
c     If in use, io(i) must be io(1)+i-1.  This subroutine sets io(1)
c     and io(2). They are not used, but they could be used for
c     a log file and a recovery file by suitable modifications of
c     subroutines in and out.
      integer io(10)
c     exit code returned on error
      integer errcod
      common/lib_cm/io,errcod

c local

c     index
      integer i

c executable

c     set the exit code returned on error to 1
      errcod=1

c     reserve two I/O units, one for the log file, one for the recovery file
      io(1)=iounit
      if(io(1).lt.1)io(1)=7
      io(2)=-1

c     mark the other I/O units as not in use.
      do 10 i=3,10
         io(i)=0
 10   continue

c     check the I/O unit
      if(iounit.lt.1)call out('*** PROGRAM ERROR: invalid I/O unit!')

      return
      end



      subroutine out(str)

c     Prints string str on the screen

c     Rings a bell when the first character of the string is '*'.

c     Terminates program if str contains an exclamation mark '!'.
c     Use subroutine out2 below if that is not desirable.

      implicit none

c input

c     string to write
      character*(*) str

c common

c     I/O units.  They are already reserved if nonzero.
c     If in use, io(i) must be io(1)+i-1
      integer io(10)
c     exit code returned on error
      integer errcod
      common/lib_cm/io,errcod

c executable

c     let out2 write the string
      call out2(str)

c     exit on '!'
      if(index(str,'!').gt.0)call errxit

      return
      end


      subroutine out2(str)

c     version of out that will not terminate the program on exclamation marks

      implicit none

c input

c     string to write
      character*(*) str

c executable

c     print the string, with bell if the first character is '*'
      if(str(1:1).eq.'*')then
         call outcon(0,1,str)
      else
         call outcon(0,0,str)
      endif

      return
      end




      subroutine in(line)

c     Reads a line from the keyboard.

      implicit none

c input/output

c     input: prompt to use.  Must be terminated by a colon.
c            Any default must follow the colon and a space or question mark
c            (question mark allows a blank input even with a default)
c     output: line read
      character*80 line

c local

c     position of colon in prompt
      integer icol

c     whether there is a default
      logical defalt

c executable

c     find colon
      icol=index(line,':')
      if(icol.le.0)
     & call out('*** IN: colon missing after prompt!')
      if(icol.gt.77)
     & call out('*** IN: prompt too long!')
      if(line(icol+1:icol+1).ne.' ' .and. line(icol+1:icol+1).ne.'?')
     & call out(
     & '*** IN: colon must be followed by a space or question mark!')
      defalt=.false.
      if(line(icol+2:80).ne.' ')defalt=.true.
      if(line(icol+1:icol+1).eq.'?')defalt=.false.

c     read from the keyboard
      call incon(line)

c     check
      if(defalt .and. line.eq.' ')
     & call out('*** IN: lost default!')

      return
      end


      subroutine in2(line,linelt)

c     Version of subroutine in above that eliminates the leading and
c     trailing whitespace.  Returns linelt=1 and line(1:1) blank if the
c     input was empty.

      implicit none

c input/output

c     input: prompt to use.  Must be terminated by a colon.
c            Any default must follow the colon and a space.
c     output: line read
      character*80 line

c     output
      integer linelt

c local

c     indices
      integer i,i2

c executable

c     read from the keyboard
      call incon(line)

c     kill trailing whitespace
      do 100 i=80,1,-1
         linelt=i
         if(line(i:i).gt.' ')goto 110
 100  continue
 110  if(linelt.le.1)return

c     kill leading whitespace
      do 200 i=1,linelt
         if(line(i:i).gt.' ')then
            if(i.eq.1)return
            linelt=linelt+1-i
            do 150 i2=1,linelt
               line(i2:i2)=line(i2+i-1:i2+i-1)
 150        continue
            return
         endif
 200  continue

      return
      end


      subroutine inkey(good,goodc,line,ans)

c     Get a key from the keyboard.

      implicit none

c input

c     accepted keys
c     1 all keys accepted (except those leading to error exit
c     2 all normal keys
c     3 all nonblank normal keys
c     4 all keys in string goodc
c     5 all keys in string goodc, key is first converted to lower case
      integer good

c     good characters, if good is 4 or 5
      character*(*) goodc

c input/output

c     input: Prompt to use.  Must be terminated by a colon.
c     output: undefined
      character*80 line

c output

c     the user input
      character*1 ans

c local

c     saved line
      character*(80) linsav

c     mode
      integer mode

c     position of the colon
      integer icol

c executable

c     save the line
      linsav=line

c     position of the colon
      icol=index(line,':')
      if(icol.le.0 .or. icol.gt.77)call out(
     & '*** INKEY: Program error: invalid prompt')
      if(line(icol+1:icol+1).ne.' ' .or. line(icol+3:80).ne.' ')
     & call out('*** INKEY: Program error: invalid prompt')

c     set the mode to initial
      mode=0

c     get the key using inconk
 1    call inconk(mode,line,ans)
      mode=1

c     examine the result
      if(good.eq.1)then
         goto 900
      else if(good.eq.2)then
         if(ans.ge.' ' .and. ans.le.'~')goto 900
         if(ans.eq.char(9))goto 900
      else if(good.eq.3)then
         if(ans.gt.' ' .and. ans.le.'~')goto 900
      else if(good.eq.4 .or. good.eq.5)then
         if(good.eq.5 .and. ans.ge.'A' .and. ans.le.'Z')
     &    ans=char(ichar(ans)+32)
         if(index(goodc,ans).gt.0)goto 900
      else
         call out('*** INKEY: Program error: invalid good!')
      endif

c     ask again
      line=linsav
      goto 1

c     tell inconk to finish
 900  call inconk(2,line,ans)

c     return the key
      return
      end



      subroutine errset(i)

c     Sets the error exit status to i.

      implicit none

c input

c     new error code
      integer i

c common

c     I/O units.  They are already reserved if nonzero.
c     If in use, io(i) must be io(1)+i-1
      integer io(10)
c     exit code returned on error
      integer errcod
      common/lib_cm/io,errcod

c executable

c     set the new code
      errcod=i

      return
      end


      subroutine errxit

c     Exits with the set error status.

      implicit none

c common

c     I/O units.  They are already reserved if nonzero.
c     If in use, io(i) must be io(1)+i-1
      integer io(10)
c     exit code returned on error
      integer errcod
      common/lib_cm/io,errcod

c executable

      call exit(errcod)

      return
      end


      subroutine opennw(lun,filnam)

c     open a new file, replacing any pre-existing one

c input

c     I/O unit number:
      integer lun
c     file name:
      character*(*) filnam

c local

c     whether the file already exists:
      logical exists

c     local version of the file name
      character*(40) filnm
      integer filnml

c     put file name in local storage (needed for gnu fortran)
      filnm=filnam
      filnml=index(filnm,' ')-1
      if(filnml.lt.1)filnml=40

c     get rid of any previous version:
      inquire(file=filnam,exist=exists,err=900)
      if(exists)then
         open(lun,file=filnam,status='old',err=910)
         close(lun,status='delete',err=910)
      endif

c     open the new version:
      open(lun,file=filnam,status='new',err=920)

      return

 900  call out('*** Unable to look for file '//filnm(1:filnml))
      call out('    check disk!')
 910  call out('*** Unable to delete file '//filnm(1:filnml))
      call out('    check file permissions and quota!')
 920  call out('*** Unable to open file '//filnm(1:filnml))
      call out('    check file permissions and quota!')
      return
      end
