      subroutine dopair(
     & reinit,verbos,iunit,fil,
     & orgdim,orgmax,orgstr,orgmod,trndim,trnmax,trnstr,
     & memdim,memmax,memstr,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,
     & fndct,subct,
     & *,*)

c     Obtain an original/ substitution pair from getsub and process it.
c     Takes exit 1 on end of unit iunit, exit 2 on reopen iunit.

c     Assumes that we are performing a sequence of substitutions on the
c     same data.  If the data changes, be sure to deselect all and/or to
c     reinitialize as appropriate.

c     avoid typos
      implicit none

c input

c     reinitialization: 0=no reinitialization, 1=reinitialize
      integer reinit
c     verbosity; 0 for no output, 1 for original and substitution printing
      integer verbos

c     unit to read the original/ translation pair from.
      integer iunit
c     file name open on the unit
      character*(*) fil

c work arrays

c     declared original string size
      integer orgdim
c     actual original string size
      integer orgmax
c     original string
      character*1 orgstr(orgdim)
c     modifiers
      character*1 orgmod(orgdim)

c     declared translation string size
      integer trndim
c     actual translation string size
      integer trnmax
c     translation string
      character*1 trnstr(trndim)

c     declared memory string size
      integer memdim
c     actual memory size (must be saved)
      integer memmax
c     memory string (must be saved)
      character*1 memstr(memdim)

c input/output

c     declared primary and secondary file buffer sizes
      integer bufdim
c     actual file buffer sizes
      integer bufmax,bufmx2
c     file buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     whether the original is in bufst2 (may not change for specials.)
      integer use2

c output:

c     counter of matches found
      integer fndct
c     counter of nontrivial substitutions done
      integer subct

c common

c     declared substring selection levels
      integer levdim
      parameter (levdim=1000)
c     actual substring selection level
      integer lev
c     available bufdim at each level
      integer bufdml(0:levdim)
c     start of the post-selection string
      integer post(levdim)

c     after a successfull \f#, the #-th occurrence is from fnddat(2) to (3)
      integer fnddat(3)

c     total substitutions
      integer subtot
c     total found strings
      integer fndtot
c     total missed substitutions
      integer subnot
c     total missed searches
      integer fndnot
c     previous value of subtot at \x command
      integer oldsct

      common/cmpair/lev,bufdml,post,
     & fnddat,
     & subtot,fndtot,subnot,fndnot,oldsct

c local

c     whether we need to initialize
      integer init
      data init/1/
      save init

c     available buffer dimension
      integer bufdm

c     initial substring selection level
      integer lev0

c     specials for the translation that just select inside dosubs
      character*5 trnspn
      data trnspn/'cfsv^'/
c     whether this is a translation special that just selects inside dosubs
      integer trnsel

c     output line
      character*1 out1(79),out2(79)

c     character codes
      integer ic0,ic

c     character index
      integer i

c executable

c     initialize
      if(init.ne.0 .or. reinit.ne.0)then
         init=0
c        make sure that no more than 123456788 substitutions are possible
         if(bufdim.gt.123456788/2-2)call fatal(
     &    'Program error: buffer size exceeds 123456788/2-2 limit')
c        clear memory
         memmax=-1
c        clear all selection levels
         lev=0
         bufdml(0)=bufdim
c        clear counts
         subtot=0
         fndtot=0
         subnot=0
         fndnot=0
         oldsct=-1
      endif

c     active buffer size
      bufdm=bufdml(lev)

c     read the next original and translation
      call getsub(iunit,fil,len(fil),orgdim,trndim,
     & orgmax,orgstr,orgmod,trnmax,trnstr)

c     exit on end of file
      if(orgmax.eq.0)return 1

c     handle the original specials

      if(orgmax.gt.0)goto 2900

      if(orgmod(1).eq.'m')then

c        take the original from memory
         if(memmax.lt.0)call fatal('Nothing stored to use as original')
         if(memmax.gt.0)then
            orgmax=memmax
            if(orgmax.gt.orgdim)call fatal('Original too long')
            do 2150 i=1,orgmax
               orgstr(i)=memstr(i)
               orgmod(i)=char(0)
 2150       continue
c        take the "not anything" string
         else
            orgmax=5
            do 2170 i=1,orgmax
               orgstr(i)=char(0)
 2170       continue
            orgmod(1)='~'
            orgmod(2)='&'
            orgmod(3)='a'
            orgmod(4)=']'
            orgmod(5)='*'
         endif

      else if(orgmod(1).eq.'d')then

c        dump -orgmax selection levels
         orgmax=-orgmax
         if(orgmax.eq.123456789)orgmax=lev
         if(orgmax.gt.lev)
     &    call fatal('There are not so many selection levels')
         lev0=lev
         do 2300 lev=lev0,lev0-orgmax+1,-1
            if(use2.eq.0)then
               bufmx2=0
               do 2210 i=bufdml(lev)+1,post(lev)-1
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2210          continue
               do 2220 i=1,bufmax
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2220          continue
               do 2230 i=post(lev),bufdml(lev-1)
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2230          continue
            else
               bufmax=0
               do 2250 i=bufdml(lev)+1,post(lev)-1
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2250          continue
               do 2260 i=1,bufmx2
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2260          continue
               do 2270 i=post(lev),bufdml(lev-1)
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2270          continue
            endif
            use2=1-use2
 2300    continue
         lev=lev0-orgmax
         return

      else if(orgmod(1).eq.'r')then

c        go reread input file
         return 2

      else if(orgmod(1).eq.'x')then

c        exit on failure to make changes after the last time we were here
         if(subtot.le.oldsct)return 1
         oldsct=subtot
         return

      else

         call fatal('Incorrect original special')

      endif

 2900 continue

c     handle some substitution specials

      trnsel=0
      if(trnmax.ge.0)goto 3900
      if(index(trnspn,trnstr(1)).gt.0)trnsel=1

c     take the substitution from memory
      if(trnstr(1).eq.'m')then
         if(memmax.lt.0)
     &    call fatal('Nothing stored to use as substitution')
         trnmax=memmax
         if(trnmax.gt.trndim)call fatal('Substitution too long')
         do 3100 i=1,trnmax
            trnstr(i)=memstr(i)
 3100    continue
      endif

 3900 continue

c     bypass substitutions if we are going to select all
      if(trnsel.ne.0 .and. orgmax.eq.3)then
         if(orgmod(1)//orgmod(2)//orgmod(3).eq.'a?a')then
            subct=0
            fnddat(1)=1
            fnddat(2)=1
            fnddat(3)=bufmax
            if(use2.ne.0)fnddat(3)=bufmx2
            goto 4900
         endif
      endif
c     do the substitutions
      if(use2.eq.0)then
         call dosubs(
     &    orgdim,orgmax,orgstr,orgmod,
     &    trndim,trnmax,trnstr,
     &    bufdm,bufmax,bufstr,
     &    bufmx2,bufst2,fnddat,subct)
      else
         call dosubs(
     &    orgdim,orgmax,orgstr,orgmod,
     &    trndim,trnmax,trnstr,
     &    bufdm,bufmx2,bufst2,
     &    bufmax,bufstr,fnddat,subct)
      endif
      if(trnsel.eq.0)use2=1-use2
 4900 fndct=fnddat(1)
      subtot=subtot+subct
      fndtot=fndtot+fnddat(1)
      if(trnmax.eq.-123456789)then
         if(fnddat(1).le.0)fndnot=fndnot+1
      else if(trnsel.ne.0)then
         if(fnddat(1).lt.-trnmax)fndnot=fndnot+1
      else
         if(fnddat(1).le.0)fndnot=fndnot+1
         subnot=subnot+fnddat(1)-subct
      endif

c     log it
      if(verbos.ne.0)then
         do 4950 i=1,min(79,orgmax)
            ic0=ichar(orgstr(i))
            ic=mod(ic0,128)
            out1(i)=' '
            out2(i)=orgstr(i)
            if(orgmod(i).eq.char(0))then
               if(ic0.ge.32 .and. ic0.le.126)goto 4950
               out1(i)='3'
               if(ic.ge.32 .and. ic.le.126)goto 4950
               out1(i)='\\'
               if(ic0.gt.127)out1(i)='/'
               if(ic.eq.0)then
                  out2(i)=' '
               else if(ic.le.26)then
                  out2(i)=char(ic+ichar('A')-1)
               else if(ic.le.29)then
                  out2(i)=char(ic+ichar('7')-27)
               else if(ic.le.31)then
                  out2(i)=char(ic+ichar('0')-30)
               else
                  out2(i)='2'
               endif
            else
               out1(i)=orgmod(i)
               if(orgmod(i).eq.'&')out1(i)='('
               if(orgmod(i).eq.'*')out1(i)=')'
               out2(i)=' '
            endif
 4950    continue
         print4960,(out1(i),i=1,min(79,orgmax))
         print4960,(out2(i),i=1,min(79,orgmax))
 4960    format(79a1)
         if(trnsel.ne.0)then
            if(trnmax.ne.-123456789)
     &       print4970,fnddat(1),subct,trnstr(1),-trnmax
            if(trnmax.eq.-123456789)
     &       print4970,fnddat(1),subct,trnstr(1)
 4970       format('   found:',i10,'   changed:',i10,
     &       '   substitution:\\',a1,i10)
         else
            do 4980 i=1,min(22,abs(trnmax))
               out1(i)=trnstr(i)
               if(out1(i).lt.' ' .or. out1(i).gt.'~')out1(i)='?'
 4980       continue
            if(trnmax.ge.0)print4990,
     &       fnddat(1),subct,' ',(out1(i),i=1,min(22,trnmax))
            if(trnmax.lt.0)print4990,
     &       fnddat(1),subct,'\\',(out1(i),i=1,min(22,-trnmax))
 4990       format('   found:',i10,'   changed:',i10,
     &       '   substitution:',23a1)
         endif
      endif

c     handle special substitutions

      if(trnsel.eq.0)goto 5900

      if(trnstr(1).eq.'s')then

c        next level of selection
         lev=lev+1
         if(lev.gt.levdim)call fatal('Too many selection levels')

c        do the selection
         if(use2.eq.0)then

c           set new buffer size and position of the post-selection string
            bufdml(lev)=bufdml(lev-1)-bufmax+fnddat(3)-fnddat(2)+1
            bufdm=bufdml(lev)
            post(lev)=bufdm+fnddat(2)

c           initialize actual selected buffer size
            bufmx2=0

c           copy the selected string in the reduced buffer
            do 5110 i=fnddat(2),fnddat(3)
               bufmx2=bufmx2+1
               bufst2(bufmx2)=bufstr(i)
 5110       continue

c           save the pre-selection string
            do 5120 i=1,fnddat(2)-1
               bufst2(bufdm+i)=bufstr(i)
 5120       continue

c           save the post-selection string
            do 5130 i=fnddat(3)+1,bufmax
               bufst2(post(lev)-fnddat(3)-1+i)=bufstr(i)
 5130       continue

c           copy the saved area to the companion
            do 5140 i=bufdm+1,bufdml(lev-1)
               bufstr(i)=bufst2(i)
 5140       continue

         else

c           set new buffer size and position of the post-selection string
            bufdml(lev)=bufdml(lev-1)-bufmx2+fnddat(3)-fnddat(2)+1
            bufdm=bufdml(lev)
            post(lev)=bufdm+fnddat(2)

c           initialize actual selected buffer size
            bufmax=0

c           copy the selected string in the reduced buffer
            do 5150 i=fnddat(2),fnddat(3)
               bufmax=bufmax+1
               bufstr(bufmax)=bufst2(i)
 5150       continue

c           save the pre-selection string
            do 5160 i=1,fnddat(2)-1
               bufstr(bufdm+i)=bufst2(i)
 5160       continue

c           save the post-selection string
            do 5170 i=fnddat(3)+1,bufmx2
               bufstr(post(lev)-fnddat(3)-1+i)=bufst2(i)
 5170       continue

c           copy the saved area to the companion
            do 5180 i=bufdm+1,bufdml(lev-1)
               bufst2(i)=bufstr(i)
 5180       continue

         endif

         use2=1-use2

      else if(trnstr(1).eq.'c')then

c        copy the selected area to memory
         if(fnddat(3)-fnddat(2)+1.gt.memdim)
     &    call fatal('memory exceeded')
         memmax=0
         if(use2.eq.0)then
            do 5210 i=fnddat(2),fnddat(3)
               memmax=memmax+1
               memstr(memmax)=bufstr(i)
 5210       continue
         else
            do 5220 i=fnddat(2),fnddat(3)
               memmax=memmax+1
               memstr(memmax)=bufst2(i)
 5220       continue
         endif

      else if(trnstr(1).eq.'^')then

c        convert the selected area to upper case
         if(use2.eq.0)then
            do 5310 i=fnddat(2),fnddat(3)
               if(bufstr(i).ge.'a' .and. bufstr(i).le.'z')then
                  subct=1
                  bufstr(i)=char(ichar(bufstr(i))-32)
               endif
 5310       continue
         else
            do 5320 i=fnddat(2),fnddat(3)
               if(bufst2(i).ge.'a' .and. bufst2(i).le.'z')then
                  subct=1
                  bufst2(i)=char(ichar(bufst2(i))-32)
               endif
 5320       continue
         endif

      else if(trnstr(1).eq.'v')then

c        convert the selected area to lower case
         if(use2.eq.0)then
            do 5350 i=fnddat(2),fnddat(3)
               if(bufstr(i).ge.'A' .and. bufstr(i).le.'Z')then
                  subct=1
                  bufstr(i)=char(ichar(bufstr(i))+32)
               endif
 5350       continue
         else
            do 5360 i=fnddat(2),fnddat(3)
               if(bufst2(i).ge.'A' .and. bufst2(i).le.'Z')then
                  subct=1
                  bufst2(i)=char(ichar(bufst2(i))+32)
               endif
 5360       continue
         endif

      else if(trnstr(1).eq.'f')then

c        do nothing

      else

         call fatal('Incorrect substitution special')

      endif

 5900 continue

c     next translation
      return

      end
