      subroutine getsub(
     & iunit,fil,fillt,orgdim,trndim,
     & orgmax,orgstr,orgmod,trnmax,trnstr)

C     Get an original and translation pair from unit iunit.

C     Each item in the original consists of an orgstr and an orgmod
C     character.  It the item is just a plain character of the original,
C     orgstr is that character and orgmod is ascii null.  On the other
C     hand, if the item is a command that cannot be represented by a
C     single character, then orgmod is that command and orgstr is ascii
C     null.

C     Returns orgmax=0 on end of file.  Returns orgmax and/or trnmax
C     negative if they are a special command.

      implicit none

C input:

C     unit to read
      integer iunit
C     file name open on it
      character*(*) fil
      integer fillt

C     declared original string size
      integer orgdim

C     declared translation string size
      integer trndim

C output:

C     actual original string size, or 0 for eof, or negative for a special
      integer orgmax
C     original string
      character*1 orgstr(orgdim)
C     modifiers
      character*1 orgmod(orgdim)

C     actual translation string size, or negative for a special
      integer trnmax
C     translation string
      character*1 trnstr(trndim)

C local:

C     whether we have reached end of file
      logical eof
      save eof
      data eof/.false./

C     line count
      integer linect
      save linect
      data linect/1/

C     single character (from a set) match specifications and char(0)
      character*10 scms
      save scms
      data scms/' ''_+56alu]'/
C     specials for the original not followed by numbers
      character*3 orgspc
      data orgspc/'mrx'/
C     specials for the original optionally followed by numbers
      character*1 orgspn
      data orgspn/'d'/
C     specials not followed by a translation
      character*3 notran
      data notran/'drx'/
C     specials for the translation not followed by numbers
      character*6 trnspc
      data trnspc/'elmrux'/
C     specials for the translation optionally followed by numbers
      character*5 trnspn
      data trnspn/'cfsv^'/
C     whether this is a special
      logical isspc

C     whether a backslash is pending
      logical bsl
C     whether a carriage return is pending
      logical cr
C     whether an 8 bit conversion is pending
      logical bit8
C     whether we are in a group
      logical group

C     file name string
      character*200 fils
      integer filslt

C     number value
      integer value

C     position in orginal string
      integer org,org2

C     position in translated string
      integer trn

C     character
      character*1 chr
      integer ichr

C executable:

C     return on end of file
      orgmax=0
      if(eof)return

C     complete scms
      scms(1:1)=char(0)

C     set file name string
      filslt=16+fillt
      if(filslt.gt.200)call fatal('Substitutions file name is too long')
      fils=' in file '
      fils(10:9+fillt)=fil(1:fillt)
      fils(10+fillt:16+fillt)=', line '

C     read the original

C     initialize the original
 1000 orgmax=0
      cr=.false.
      bsl=.false.
      bit8=.false.

C     read from the file
 1100 call getchr(iunit,chr,*1110,*1120)
      goto 1130
 1110 eof=.true.
      if(cr)call fatali('Carriage return'//fils(1:filslt),linect)
      if(bsl)call fatali('Incomplete \\ command at eof'//
     & fils(1:filslt),linect)
      if(bit8)call fatali(
     & 'Incomplete "\\3" command'//fils(1:filslt),linect)
      linect=linect+1
      if(orgmax.le.0)return
      goto 1900
 1120 call fatal2('Disk error reading file ',fil(1:fillt))
 1130 continue

C     check for end of line
      if(chr.eq.char(10))then
         if(bsl)call fatali(
     &    'Incomplete \\ command at new line'//fils(1:filslt),linect)
         if(bit8)call fatali(
     &    'Incomplete "\\3" command'//fils(1:filslt),linect)
         linect=linect+1
         goto 1900
      endif

C     carriage control character check
      if(cr)call fatali('Carriage return'//fils(1:filslt),linect)
      cr=.false.
      if(chr.eq.char(13))then
         cr=.true.
         goto 1100
      endif

C     increment the buffer
      orgmax=orgmax+1
      if(orgmax.gt.orgdim)call fatali(
     & 'String to translate too long'//fils(1:filslt),linect)

C     process the character based on whether we are in a backslash sequence
      orgmod(orgmax)=char(0)
      if(bsl)then
         bsl=.false.
C        create special characters
         if(chr.eq.'\\')then
         elseif(chr.eq.' ')then
            chr=char(0)
         elseif(chr.ge.'A' .and. chr.le.'Z')then
            chr=char(ichar(chr)-ichar('A')+1)
         elseif(chr.ge.'7' .and. chr.le.'9')then
            chr=char(ichar(chr)-ichar('7')+27)
         elseif(chr.ge.'0' .and. chr.le.'1')then
            chr=char(ichar(chr)-ichar('0')+30)
         elseif(chr.eq.'2')then
            chr=char(127)
         elseif(chr.eq.'3')then
            chr=char(0)
            bit8=.true.
         else
            if(bit8)call fatali(
     &       'Incomplete "\\3" command'//fils(1:filslt),linect)
            orgmod(orgmax)=chr
            chr=char(0)
         endif
      else
C        check for a new backslash sequence
         if(chr.eq.'\\')then
            bsl=.true.
            orgmax=orgmax-1
            goto 1100
         endif
      endif

C     add the character
      orgstr(orgmax)=chr
      if(bit8)then
         bit8=.false.
         orgmax=orgmax-1
         ichr=ichar(chr)+128
         if(ichr.gt.255)call fatali(
     &    'Illegal 8 bit character'//fils(1:filslt),linect)
         orgstr(orgmax)=char(ichr)
      endif
      goto 1100

C     ignore empty lines before originals
 1900 if(orgmax.le.0)goto 1000

C     check the original for consistency
      group=.false.
      do 1950 org=1,orgmax
         chr=orgmod(org)
C        single character match specifications are fine
         if(index(scms,chr).gt.0)goto 1950
C        translate aliases
         if(index('()',chr).gt.0)then
            if(chr.eq.'(')chr='&'
            if(chr.eq.')')chr='*'
            orgmod(org)=chr
         endif
C        case changes are fine
         if(index('-=',chr).gt.0)goto 1950
C        no number specials are only fine alone.
         if(index(orgspc,chr).gt.0)then
            if(orgmax.ne.1)call fatal('Illegal original special: '//chr)
            orgmax=-1
            goto 1951
         endif
C        number specials are only fine at the start and followed by digits
         if(index(orgspn,chr).gt.0)then
            if(org.ne.1)call fatal('Illegal original special: '//chr)
            value=0
            do 1920 org2=2,orgmax
               if(orgstr(org2).lt.'0' .or. orgstr(org2).gt.'9')call
     &          fatal('Invalid number behind original special '//chr)
               value=value*10+ichar(orgstr(org2))-48
 1920       continue
            if(value.ge.123456789)call fatal(
     &       'Original special '//chr//' has a too large count')
            orgmax=-value
            if(orgmax.eq.0)orgmax=-123456789
            goto 1951
         endif
C        check groups
         if(group)then
            if(index('&?~',chr).gt.0)call fatali(
     &       'Illegal "\\'//chr//'" in group'//fils(1:filslt),linect-1)
         else
            if(chr.eq.'*')call fatali(
     &       'No group for "\\*" to terminate'//fils(1:filslt),linect-1)
         endif
         if(chr.eq.'&')then
            group=.true.
            goto 1950
         endif
         if(chr.eq.'*')then
            group=.false.
            goto 1950
         endif
C        check \? for preceding a \~, group or normal character
         if(chr.eq.'?')then
            if(index('~&('//scms,orgmod(min(orgmax,org+1))).gt.0)
     &       goto 1950
            call fatali(
     &       'Incomplete command "\\?"'//fils(1:filslt),linect-1)
C        check \~ for preceding a group or normal character
         elseif(chr.eq.'~')then
            if(index('&('//scms,orgmod(min(orgmax,org+1))).gt.0)
     &       goto 1950
            call fatali(
     &       'Incomplete command "\\~"'//fils(1:filslt),linect-1)
         else
            call fatali(
     &       'Illegal command "\\'//chr//'"'//fils(1:filslt),linect-1)
         endif
 1950 continue
 1951 if(group)call fatali(
     & 'Unterminated "\\&" group'//fils(1:filslt),linect)

C     return if there is no translation
      if(index(notran,orgmod(1)).gt.0)return

C     read the translation

C     initialize the translation
      trnmax=0
      cr=.false.
      bsl=.false.
      bit8=.false.
      isspc=.false.

C     read from the file
 2100 call getchr(iunit,chr,*2110,*2120)
      goto 2130
 2110 eof=.true.
      if(cr)call fatali('Carriage return'//fils(1:filslt),linect)
      if(bsl)call fatali('Incomplete \\ command'//fils(1:filslt),linect)
      if(bit8)call fatali(
     & 'Incomplete "\\3" command'//fils(1:filslt),linect)
      linect=linect+1
      goto 2900
 2120 call fatal2('Disk error reading file ',fil(1:fillt))
 2130 continue

C     check for end of line
      if(chr.eq.char(10))then
         if(bsl)call fatali(
     &    'Incomplete \\ command'//fils(1:filslt),linect)
         if(bit8)call fatali(
     &    'Incomplete "\\3" command'//fils(1:filslt),linect)
         linect=linect+1
         goto 2900
      endif

C     carriage control character check
      if(cr)call fatali('Carriage return'//fils(1:filslt),linect)
      cr=.false.
      if(chr.eq.char(13))then
         cr=.true.
         goto 2100
      endif

C     increment the buffer
      trnmax=trnmax+1
      if(trnmax.gt.trndim)call fatali(
     & 'Substitution string too long'//fils(1:filslt),linect)

C     process the character based on whether we are in a backslash sequence
      if(bsl)then
         bsl=.false.
C        create special characters
         if(chr.eq.'\\')then
         elseif(chr.eq.' ')then
            chr=char(0)
         elseif(chr.ge.'A' .and. chr.le.'Z')then
            chr=char(ichar(chr)-ichar('A')+1)
         elseif(chr.ge.'7' .and. chr.le.'9')then
            chr=char(ichar(chr)-ichar('7')+27)
         elseif(chr.ge.'0' .and. chr.le.'1')then
            chr=char(ichar(chr)-ichar('0')+30)
         elseif(chr.eq.'2')then
            chr=char(127)
         elseif(chr.eq.'3')then
            chr=char(0)
            bit8=.true.
         elseif(index(trnspc//trnspn,chr).gt.0)then
            if(trnmax.ne.1)
     &       call fatal('Misplaced substitution special: '//chr)
            isspc=.true.
         elseif(index(scms//'-=&*?~',chr).gt.0)then
            call fatali('Misplaced command "\\'//chr//'"'//
     &       fils(1:filslt),linect)
         else
            call fatali(
     &       'Invalid command "\\'//chr//'"'//fils(1:filslt),linect)
         endif
      else
C        check for a new backslash sequence
         if(chr.eq.'\\')then
            bsl=.true.
            trnmax=trnmax-1
            goto 2100
         endif
      endif

C     add the character
      trnstr(trnmax)=chr
      if(bit8)then
         bit8=.false.
         trnmax=trnmax-1
         ichr=ichar(chr)+128
         if(ichr.gt.255)call fatali(
     &    'Illegal 8 bit character'//fils(1:filslt),linect)
         trnstr(trnmax)=char(ichr)
      endif
      goto 2100

C     check for specials
 2900 if(isspc)then
         chr=trnstr(1)
         if(trnmax.ge.123456789)call fatal(
     &    'Too many characters behind substitution special '//chr)
         if(index(trnspc,chr).gt.0)then
            if(chr.eq.'e')then
               if(trnmax.lt.3)call fatal('Substitution special e '//
     &          'must be followed by at least 2 characters')
            else if(chr.eq.'r')then
               if(trnmax.lt.2)call fatal('Substitution special r '//
     &          'must be followed by at least one character')
            else if(trnmax.ne.1)then
               call fatal(
     &          'Substitution special '//chr//' must be alone')
            endif
            trnmax=-trnmax
         else if(index(trnspn,chr).gt.0)then
            value=0
            do 2950 trn=2,trnmax
               if(trnstr(trn).lt.'0' .or. trnstr(trn).gt.'9')call fatal(
     &          'Invalid number behind substitution special '//chr)
               if(value.ge.12345679)call fatal(
     &          'Substitution special '//chr//' has a too large count')
               value=value*10-48+ichar(trnstr(trn))
 2950       continue
            if(value.ge.123456789)call fatal(
     &       'Substitution special '//chr//' has a too large count')
            trnmax=-value
            if(trnmax.eq.0)trnmax=-123456789
         else
            call fatal('Program error in getsub')
         endif
      endif

      return
      end
