      subroutine openru(filnam,iunit)

c     Open a file for unformatted reading.

      implicit none

c input:

c     file to open
      character*(*) filnam

c     unit to open it on
      integer iunit

c local:

c     whether the file exists
      logical exists

c executable:

c     see whether the file exists
      inquire(file=filnam,exist=exists,err=10)
      goto 20
 10   call fatal2('Disk error looking for file ',filnam)
 20   if(.not.exists)call fatal2('File not found: ',filnam)

c     open it
      open(iunit,file=filnam,status='old',form='unformatted',err=30)
      goto 40
 30   call fatal2('Unable to open file ',filnam)
 40   continue

      return
      end



      subroutine openwu(filnam,iunit)

c     Open a file for unformatted writing.

      implicit none

c input:

c     file to open
      character*(*) filnam

c     unit to open it on
      integer iunit

c local:

c     whether the file exists
      logical exists

c executable:

c     see whether the file exists
      inquire(file=filnam,exist=exists,err=10)
      goto 20
 10   call fatal2('Disk error looking for file ',filnam)
 20   if(.not.exists)goto 100

c     open it
      open(iunit,file=filnam,status='old',err=30)
      goto 40
 30   call fatal2('Unable to open file ',filnam)
 40   continue

c     delete it
      close(iunit,status='delete',err=50)
      goto 60
 50   call fatal2('Unable to delete the old file ',filnam)
 60   continue

c     open it again
 100  open(iunit,file=filnam,status='unknown',
     & form='unformatted',err=110)
      goto 120
 110  call fatal2('Unable to open file ',filnam)
 120  continue

      return
      end


      subroutine closu(iunit,filnam)

c     Close unit iunit.

      implicit none

c input:

c     unit to close
      integer iunit

c     file name open on it
      character*(*) filnam

c executable:

c     close the file
      close(iunit,err=10)
      goto 20
 10   call fatal2('Disk error closing file ',filnam)
 20   continue

      return
      end



      subroutine readfu(fil,fillt,iunit,bufdim,bufmax,bufstr)

c     Reads file fil into buffer bufstr using unformatted I/O.

      implicit none

c input:

c     name of file to read
      character*(*) fil
      integer fillt

c     unit to use
      integer iunit

c     declared buffer string size
      integer bufdim

c output:

c     actual buffer string size
      integer bufmax
c     actual buffer size
      character*1 bufstr(bufdim)

c local:

c     read character
      character*1 chr

c executable:

c     open the file
      call openru(fil(1:fillt),iunit)

c     initialize the buffer size
      bufmax=0

c     read the next character
 100  call getchr(iunit,chr,*900,*110)
      goto 200
 110  call  fatal2('Error reading file ',fil(1:fillt))

c     put the character in the buffer
 200  bufmax=bufmax+1
      if(bufmax.gt.bufdim)call fatal2(
     & 'File is too big to process: ',fil(1:fillt))
      bufstr(bufmax)=chr

c     next character
      goto 100

c     close the file
 900  call closu(iunit,fil(1:fillt))

      return
      end



      subroutine writfu(fil,fillt,iunit,bufdim,bufmax,bufstr)

c     Writes buffer bufstr to file fil using unformatted I/O.

      implicit none

c input:

c     name of file to write
c        if fillt=0, the file is neither opened nor closed.
c        if fillt=-1, the file is closed but not deleted on failure
      character*(*) fil
      integer fillt

c     unit to use
      integer iunit

c     declared buffer string size
      integer bufdim
c     actual buffer string size
      integer bufmax
c     buffer string
      character*1 bufstr(bufdim)

c local:

c     buffer position
      integer buf

c executable:

c     open the file
      if(fillt.gt.0)call openwu(fil(1:fillt),iunit)

c     write the buffer
      do 200 buf=1,bufmax
         call putchr(iunit,bufstr(buf),*990)
 200  continue

c     close the file
      if(fillt.gt.0 .or. fillt.eq.-1)close(iunit,err=990)

c     done
      return

c     try to delete the file
 990  if(fillt.le.0)call fatali(
     & 'Unable to close a file.  Check free disk space.  I/O: ',iunit)
      close(iunit,status='delete',err=991)
 991  call  fatal2('Error writing to file ',fil(1:fillt))

      end
