#!/usr/local/bin/perl -w

# ********************* PROGRAM IS UNTESTED ****************************

# Purpose: Correct the bounding box in a postscript file generated
#          by latex2html.
#
# This file is for unix operating system.  You need gs (ghostscript)
# in your path.
#
# Usage: perl [path]l2hpsfbbux.pl image<nnn>.ps
#        where <nnn> is the number of the ps file to correct.  The
#        file with the corrected bounding box will be called
#        image<nnn>.eps and should be replacing the figure in the
#        tex source, through using \epsffile{image<nnn>.eps}, assuming
#        package epsf is used.
#
# Based on the perl script psfbb written by Stephen Eglen, University
# of Cambridge http://www.anc.ed.ac.uk/~stephen/postscript/psfbb

# Get the input, output, and temporary file names
if($#ARGV<0)
{
    # No arguments supplied.  
    die 'Usage: perl [path]l2hpsfbbux.pl image<nnn>.ps';
}
elsif ($#ARGV==0)
{
    # One argument supplied
    $psfile = $ARGV[0];
    if (!-f $psfile) {die "File $psfile was not found!";}
    $newfile = $psfile;
    if (!($newfile =~ s|\.ps$|.eps|)) {die "File $psfile is not a ps file!";}
    $tmpfile = $psfile;
    if (!($tmpfile =~ s|\.ps$|.tmp|)) {die "File $psfile is not a ps file!";}
}
else {
    # Too many arguments supplied.  
    die 'Usage: perl [path]l2hpsfbbux.pl image<nnn>.ps';
}
print "l2hpsfbbux: input '$psfile' output '$newfile' temp '$tmpfile'\n";

# Run gs to find the proper bounding box.  The output is normally two
# lines long, one with `BoundingBox' and one with `HiResBoundingBox'.
# The stderr redirection unfortunately does not work on Windows 98
`gs -dBATCH -dNOPAUSE -q -sDEVICE=bbox $psfile 2>$tmpfile`;

# get the ptoper bounding box out of the tmpfile
if (!open(IN, "$tmpfile")) {die "No $tmpfile, check gs!";}
$bb = <IN>;
if (!close(IN)) {die "Cannot close $tmpfile??";}
unlink($tmpfile);
print "Ghostscript gives '$bb'\n";
if (!($bb =~ m/^%%BoundingBox: /))
{die "Got no bounding box from ghostscript!";}

# Open the old and new files
if (!open(IN, "$psfile")) {die "Could not open $psfile";}
if (!open(OUT, ">$newfile")) {die "Could not open output file $newfile";}

# Copy the input file to the output one, just changing the bounding box
# on the way.
# Note, if the input file doesn't have the `BoundingBox' comment, we 
# don't replace it with the new one!!!
while(<IN>)
{
    if (/^%%BoundingBox:/) {print OUT "$bb\n"; $bb='';}
    else {print OUT;}
}

# Warn if there was no bounding box to change
if ($bb)
{
    unlink("$newfile");
    die "There was no bounding box in the file to change";
}

# Success
print "EPS file $newfile created\n";

########################################################


__END__

=head1 NAME

l2hpsfbbux.pl - Fix the bounding box within a latex2html postscript file
                due to a poor tex source file.

=head1 SYNOPSIS

perl [path]l2hpsfbbux.pl image<nnn>.ps

=head1 DESCRIPTION

This script uses ghostscript to find the correct bounding box.  It then
duplicates the file, but inserting the proper bounding box along the way.

=head1 OPTIONS

=head1 ENVIRONMENT

=head1 EXAMPLES

The following command creates image123.eps from image123.ps:

perl [path]l2hpsfbbux.pl image123.ps

=head1 FILES

=head1 AUTHOR

Leon van Dommelen <dommelen@eng.fsu.edu> based on the psfbb script ny
Stephen Eglen <stephen@anc.ed.ac.uk>.

=head1 SEE ALSO

=head1 DIAGNOSTICS

=head1 BUGS

This script assumes that the file has a bounding box, however faulty.
If there is no bounding box, latex2html does not have any problem
using the file as is anyway.  But you could manually insert the
listed bounding box in the file after the CreationDate comment line
if you want a proper eps file.

=cut
