      subroutine jpegsz(iunit,filnam,nx,ny)

c     Purpose: Find the size of a jpeg file.

c     Requires subroutine getchr.

c     avoid typos
      implicit none

c input:

c     free I/O unit to be used
      integer iunit
c     file name of the jpeg
      character*(*) filnam

c output:

c     size of the image in pixels
c     nx is set to -1 if the size could not be found
c     nx is set to -2 if the file could not be opened
c     nx is set to -3 if the file could not be found
      integer nx,ny

c local:

c     number of jpeg markers
      integer mrkdim
      parameter (mrkdim=10)
c     markers (http://www.obrador.com/essentialjpeg/HeaderInfo.htm)
      character*2 mrkstr(0:mrkdim)
      data mrkstr/
c     start of image
     & 'D8',
c     JFIF marker
     & 'E0',
c     quantization table
     & 'DB',
c     Huffman table
     & 'C4',
c     start of frame
     & 'C0',
c     start of scan
     & 'DA',
c     comment
     & 'FE',
c     unknown to leon
     & 'E1',
     & 'ED',
     & 'EE',
c     end of image
     & 'D9'/
c     markers as characters
      character*1 mrkchr(0:mrkdim)
c     marker index
      integer mrk
c     start of frame marker
      integer mrksof
      parameter (mrksof=4)

c     whether a file exist
      logical exists

c     number of characters to skip
      integer skip

c     index
      integer i

c     characters
      character*1 chr1,chr2
      integer ichr

c executable:

c     convert the markers in characters
      do 100 mrk=0,mrkdim
         if(mrkstr(mrk)(1:1).ge.'A'.and.mrkstr(mrk)(1:1).le.'F')then
            ichr=ichar(mrkstr(mrk)(1:1))-ichar('A')+10
         elseif(mrkstr(mrk)(1:1).ge.'0'.and.mrkstr(mrk)(1:1).le.'9')then
            ichr=ichar(mrkstr(mrk)(1:1))-ichar('0')
         else
            goto 990
         endif
         if(mrkstr(mrk)(2:2).ge.'A'.and.mrkstr(mrk)(2:2).le.'F')then
            ichr=ichr*16+ichar(mrkstr(mrk)(2:2))-ichar('A')+10
         elseif(mrkstr(mrk)(2:2).ge.'0'.and.mrkstr(mrk)(2:2).le.'9')then
            ichr=ichr*16+ichar(mrkstr(mrk)(2:2))-ichar('0')
         else
            goto 990
         endif
         mrkchr(mrk)=char(ichr)
 100  continue

c     initialize the output
      nx=-3
      ny=0

c     see whether the file exist
      inquire(file=filnam,exist=exists)
      if(.not.exists)return
      nx=-2

c     open the file
      open(iunit,file=filnam,status='old',form='unformatted',err=210)
      goto 220
 210  return
 220  nx=-1

c     read the start of image marker
      call getchr(iunit,chr1,*910,*920)
      call getchr(iunit,chr2,*910,*920)
      if(chr1//chr2.ne.char(255)//mrkchr(0))goto 930

c     read through the blocks to the start of image one
 500  call getchr(iunit,chr1,*910,*920)
      call getchr(iunit,chr2,*910,*920)
c     find the marker in the list
      do 600 mrk=1,mrkdim-1
         if(chr1//chr2.ne.char(255)//mrkchr(mrk))goto 600
         call getchr(iunit,chr1,*910,*920)
         call getchr(iunit,chr2,*910,*920)
         if(mrk.ne.mrksof)then
            skip=ichar(chr1)*256+ichar(chr2)
            if(skip.lt.2)goto 930
            do 520 i=1,skip-2
               call getchr(iunit,chr1,*910,*920)
 520        continue
            goto 500
         endif
         call getchr(iunit,chr1,*910,*920)
         call getchr(iunit,chr1,*910,*920)
         call getchr(iunit,chr2,*910,*920)
         ny=ichar(chr1)*256+ichar(chr2)
         call getchr(iunit,chr1,*910,*920)
         call getchr(iunit,chr2,*910,*920)
         nx=ichar(chr1)*256+ichar(chr2)
         goto 900
 600  continue
c     check for end of image marker
      if(chr1//chr2.eq.char(255)//mrkchr(mrkdim))goto 900
c     unknown marker?
      print*,'unknown marker',ichar(chr1),ichar(chr2)
      call getchr(iunit,chr1,*910,*920)
      call getchr(iunit,chr2,*910,*920)
      skip=ichar(chr1)*256+ichar(chr2)
      if(skip.lt.2)goto 930
      do 620 i=1,skip-2
         call getchr(iunit,chr1,*910,*920)
 620  continue
      goto 500

c     close the file
 900  close(iunit)
      return
c     premature end of file
 910  close(iunit)
      ny=-1
      return
c     read error
 920  close(iunit)
      ny=-2
      return
c     jpeg format error
 930  close(iunit)
      ny=-3
      return
c     program error
 990  print*,'*** program error in jpegsz!'
      stop
      end



      subroutine gifsz(iunit,filnam,nx,ny)

c     Purpose: Find the size of a gif 87a/89a file.

c     avoid typos
      implicit none

c input:

c     free I/O unit to be used
      integer iunit
c     file name of the gif
      character*(*) filnam

c output:

c     size of the image in pixels
c     nx is set to -1 if the size could not be found
c     nx is set to -2 if the file could not be opened
c     nx is set to -3 if the file could not be found
      integer nx,ny

c local:

c     whether a file exist
      logical exists

c     character
      character*1 chr

c executable:

c     initialize the output
      nx=-3
      ny=0

c     see whether the file exist
      inquire(file=filnam,exist=exists)
      if(.not.exists)return
      nx=-2

c     open the file
      open(iunit,file=filnam,status='old',form='unformatted',err=210)
      goto 220
 210  return
 220  nx=-1

c     read the start of image marker
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'G')goto 930
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'I')goto 930
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'F')goto 930
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'8')goto 930
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'7' .and. chr.ne.'9')goto 930
      call getchr(iunit,chr,*910,*920)
      if(chr.ne.'a')goto 930

c     read width
      call getchr(iunit,chr,*910,*920)
      nx=ichar(chr)
      call getchr(iunit,chr,*910,*920)
      nx=nx+ichar(chr)*256

c     read height
      call getchr(iunit,chr,*910,*920)
      ny=ichar(chr)
      call getchr(iunit,chr,*910,*920)
      ny=ny+ichar(chr)*256

c     close the file
 900  close(iunit)
      return
c     premature end of file
 910  close(iunit)
      ny=-1
      return
c     read error
 920  close(iunit)
      ny=-2
      return
c     gif format error
 930  close(iunit)
      ny=-3
      return
      end
