      subroutine selatt(tag,att,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,indent)

c     Selects a HTML tag attribute from the index.css buffer.

c     If the tag does not exist, returns fndct=-1.
c
c     If the tag exists, but the attribute does not, returns fnct=0 and
c     selects from the last nonwhite character to the tag's closing
c     curly bracket.
c
c     If the attribute exists, selects from the character immediately
c     before the attribute to its closing semicolon or the tag's curly
c     closing bracket.
c
c     avoid typos
      implicit none

c input

c     tag to find
      character*(*) tag

c     attribute of that tag to find
      character*(*) att

c input/output

c     declared primary and secondary file buffer sizes
      integer bufdim
c     actual file buffer sizes
      integer bufmax,bufmx2
c     buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     whether the original is in bufst2
      integer use2

c output

c     number of occurrences of the attribute
      integer fndct
c     indentation of the opening curly bracket
      integer indent

c local

c     local copy of the attribute, to keep g77 happy
      character*80 latt
      integer lattl

c     index
      integer i

c     temporaries
      integer itemp,itemp2

c executable:

c     initialize the indentation
      indent=0

c     copy the attribute to local
      lattl=len(att)
      if(lattl.le.0)call fatal('selatt: Missing attribute')
      if(lattl.gt.80)call fatal('selatt: Attribute too long')
      latt(1:lattl)=att

c     select the tag
      call seltag(tag,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct)
      if(fndct.le.0)then
         fndct=-1
         return
      endif

c     find the white space
      if(use2.eq.0)then
         do 100 i=1,bufmax
            if(bufstr(i).eq.'{')then
               goto 190
            else if(bufstr(i).eq.char(10))then
               indent=0
            else if(bufstr(i).eq.char(13))then
               indent=0
            else
               indent=indent+1
               if(bufstr(i).eq.char(9))indent=(indent+7)/8*8
            endif
 100     continue
      else
         do 150 i=1,bufmx2
            if(bufst2(i).eq.'{')then
               goto 190
            else if(bufst2(i).eq.char(10))then
               indent=0
            else if(bufst2(i).eq.char(13))then
               indent=0
            else
               indent=indent+1
               if(bufst2(i).eq.char(9))indent=(indent+7)/8*8
            endif
 150     continue
      endif
 190  continue

c     select the attribute, including the preceding character
      call trnit(
     & '\\-\\~\\&\\l-:\\*'//latt(1:lattl)//':\\?\\~\\&;}\\*\\&;}\\*',
     & '\\s',bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)

c     warn for multiple instances
      if(fndct.gt.1)then
         call type('dupatt')
         call out2('      '//latt(1:lattl))
         call out2('   in tag:')
         print210,tag
 210     format('      ',a)
         call typep('restidx')
      endif

c     if none, select from the last nonblank character to the closing bracket
      if(fndct.eq.0)then
         call trnit(
     &    '\\d1',' ',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,itemp,itemp2)
         call trnit(
     &    '\\~\\+\\?\\+}','\\s',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,itemp,itemp2)
      endif

c     all done
      return
      end
