      subroutine setval(tag,att,val,create,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)

c     Sets a HTML tag attribute value in the index.css buffer.

c     returns fndct=-1 if the tag does not exist, 0 if it does, but the
c     value could not be set.

c     avoid typos
      implicit none

c input

c     tag to change
      character*(*) tag

c     attribute of that tag to change/create
      character*(*) att

c     value to give the attribute
      character*(*) val

c     nonzero to create the attribute if it does not yet exist
      integer create

c input/output

c     declared primary and secondary file buffer sizes
      integer bufdim
c     actual file buffer sizes
      integer bufmax,bufmx2
c     buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     whether the original is in bufst2
      integer use2

c output

c     number of occurrences of the attribute
      integer fndct
c     nonzero if a change was made
      integer subct

c common data from dopair

c     declared substring selection levels
      integer levdim
      parameter (levdim=1000)
c     actual substring selection level
      integer lev
c     available bufdim at each level
      integer bufdml(0:levdim)
c     start of the post-selection string
      integer post(levdim)
c     after a successfull \f#, the #-th occurrence is from fnddat(2) to (3)
      integer fnddat(3)
c     total substitutions
      integer subtot
c     total found strings
      integer fndtot
c     total missed substitutions
      integer subnot
c     total missed searches
      integer fndnot
c     previous value of subtot at \x command
      integer oldsct

      common/cmpair/lev,bufdml,post,
     & fnddat,
     & subtot,fndtot,subnot,fndnot,oldsct

c local

c     length of the attribute
      integer attl

c     length of value
      integer vall

c     white space
      character*4 white

c     indentation of the opening curly bracket
      integer indent

c     indentation string
      character*80 ins
      integer insl

c     shift
      integer shift

c     index
      integer i
c     limits of the value
      integer i1,i2

c     temporaries
      integer itemp,itemp2

c executable:

c     white space
      white=' '//char(9)//char(10)//char(13)

c     set the length of value
      vall=len(val)
      if(vall.le.0)call fatal('setval: Value has no length')

c     initialize the substitution count
      subct=0

c     select the attribute
      call selatt(tag,att,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,indent)
      if(fndct.lt.0)goto 900

c     restrict us first to the case of an existing attribute
      if(fndct.eq.0)goto 500

c     select the value from : to closing semicolon or bracket
      call trnit(
     & ':\\?\\~\\&;}\\*\\&;}\\*','\\s1',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)
      if(fndct.lt.1)call fatal('setval: Program error')

c     ensure the selection is in bufstr to simplify things
      call set1(bufdim,bufmax,bufstr,bufmx2,bufst2,use2)

      if(bufstr(bufmax).eq.';')then

c        just replace everything to the semicolon

c        new buffer length
         if(bufmax.ne.3+vall)subct=1
         bufmax=3+vall
         if(bufmax.gt.bufdml(lev))call fatal('setval: Out of storage')

c        insert
         if(bufstr(2).ne.' ')subct=1
         bufstr(2)=' '
         do 110 i=1,vall
            if(bufstr(2+i).ne.val(i:i))subct=1
            bufstr(2+i)=val(i:i)
 110     continue
         bufstr(bufmax)=';'

      else

c        maintain the final whitespace

c        find the upper limit of the value without trailing whitespace
         do 140 i=bufmax-1,1,-1
            i2=i
            if(index(white,bufstr(i)).le.0)goto 141
 140     continue
 141     continue

c        the amount to shift the trailing edge
         shift=2+vall-i2
         if(bufmax+shift.gt.bufdml(lev))
     &    call fatal('setval: Out of storage')

c        shift it
         if(shift.lt.0)then
            subct=1
            do 150 i=i2+1,bufmax
               bufstr(i+shift)=bufstr(i)
 150        continue
         else if(shift.gt.0)then
            subct=1
            do 160 i=bufmax,i2+1,-1
               bufstr(i+shift)=bufstr(i)
 160        continue
         endif
         bufmax=bufmax+shift

c        insert the new value
         if(bufstr(2).ne.' ')subct=1
         bufstr(2)=' '
         do 170 i=1,vall
            if(bufstr(i+2).ne.val(i:i))subct=1
            bufstr(i+2)=val(i:i)
 170     continue

      endif

c     done
      goto 900

c     create the attribute
 500  if(create.eq.0)goto 900

c     we have "found" the attribute and changed it
      fndct=1
      subct=1

c     ensure the selection is in bufstr to simplify things
      call set1(bufdim,bufmax,bufstr,bufmx2,bufst2,use2)

c     set the attribute length
      attl=len(att)

c     find the indentation string
      insl=indent/8+mod(indent,8)
      if(insl.gt.80)call fatal('setval: Too much indentation')
      do 510 i=1,indent/8
         ins(i:i)=char(9)
 510  continue
      do 520 i=indent/8+1,insl
         ins(i:i)=' '
 520  continue

c     add a closing semi-colon if appropriate
      i1=1
      if(index('{;',bufstr(1)).eq.0)then
         i1=2
         bufstr(2)=';'
      endif

c     add tag and value
      bufmax=i1+1+insl+attl+2+vall+1+insl+1
      if(bufmax.gt.bufdml(lev))call fatal('setval: Out of storage')
      i1=i1+1
      bufstr(i1)=char(10)
      do 530 i=1,insl
         i1=i1+1
         bufstr(i1)=ins(i:i)
 530  continue
      do 540 i=1,attl
         i1=i1+1
         bufstr(i1)=att(i:i)
 540  continue
      i1=i1+1
      bufstr(i1)=':'
      i1=i1+1
      bufstr(i1)=' '
      do 550 i=1,vall
         i1=i1+1
         bufstr(i1)=val(i:i)
 550  continue
      i1=i1+1
      bufstr(i1)=char(10)
      do 560 i=1,insl
         i1=i1+1
         bufstr(i1)=ins(i:i)
 560  continue
      i1=i1+1
      bufstr(i1)='}'

c     deselect all
 900  call trnit(
     & '\\d',' ',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,itemp,itemp2)

c     all done
      return
      end
